--***************************************************************
--* File         : TB_DUMPHEX.vhd                                *
--* Created      : time-stamp: <08/03/03 02:28:50 kawasan>      *
--* Modified     : Time-stamp: <08/04/16 23:46:39 kawasan>      *
--*								*
--* Test-Bench                                                  *
--*								*
--* Copyright 2008 Osamu Kawashima. all rights reserved		*
--***************************************************************

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;

use std.textio.all;
use ieee.std_logic_textio.all;

entity TB_DUMPHEX is end;

architecture behavior of TB_DUMPHEX is
    
	component DUMPHEX
	generic (
		SADD_SIZE       : integer   := 32
	);
	port (
		RESET           : in  std_logic;
		SYSCLK          : in  std_logic;

		S_SADD          : in  std_logic_vector(SADD_SIZE-1 downto 0);
		S_EADD          : in  std_logic_vector(SADD_SIZE-1 downto 0);
		GO              : in  std_logic;
		DONE            : out std_logic;

		S_ADD           : out std_logic_vector(SADD_SIZE-1 downto 0);
		S_DI            : in  std_logic_vector(7 downto 0);
		S_GO            : out std_logic;
		S_DONE          : in  std_logic;

		D_DO            : out std_logic_vector(7 downto 0);
		D_GO            : out std_logic;
		D_DONE          : in  std_logic
	);
	end component;

	constant ZERO32         : std_logic_vector(31 downto 0) := (others=>'0');
	constant ONE32          : std_logic_vector(31 downto 0) := (others=>'1');

	constant SADD_SIZE      : integer := 32;

	constant WT1            : Time := 10 ns;
	constant WT_DELAY       : Time := 1 ns;
	constant WT_IDLE        : Time := 100 ns;
	constant CLK_CYCLE      : Time := 21 ns;	-- about 48MHz

	signal RESET            : std_logic := '1';
	signal SYSCLK           : std_logic := '0';
	signal S_SADD           : std_logic_vector(SADD_SIZE-1 downto 0) := (others=>'0');
	signal S_EADD           : std_logic_vector(SADD_SIZE-1 downto 0) := (others=>'0');
	signal GO               : std_logic := '0';
	signal DONE             : std_logic := '0';
	signal S_ADD            : std_logic_vector(SADD_SIZE-1 downto 0) := (others=>'0');
	signal S_DI             : std_logic_vector(7 downto 0) := (others=>'0');
	signal S_GO             : std_logic := '0';
	signal S_DONE           : std_logic := '1';
	signal D_DO             : std_logic_vector(7 downto 0) := (others=>'0');
	signal D_GO             : std_logic := '0';
	signal D_DONE           : std_logic := '1';

	signal s_di_cnt         : std_logic_vector(7 downto 0) := (others=>'0');

begin
    
	uDUMPHEX : DUMPHEX
	generic map (
		SADD_SIZE       => SADD_SIZE
	)
	port map (
		RESET           => RESET,
		SYSCLK          => SYSCLK,

		S_SADD          => S_SADD,
		S_EADD          => S_EADD,
		GO              => GO,
		DONE            => DONE,

		S_ADD           => S_ADD,
		S_DI            => S_DI,
		S_GO            => S_GO,
		S_DONE          => S_DONE,

		D_DO            => D_DO,
		D_GO            => D_GO,
		D_DONE          => D_DONE
	);

	process begin
		SYSCLK <= '0';
		wait for CLK_CYCLE/2;
		SYSCLK <= '1';
		wait for CLK_CYCLE/2;
	end process;

	-- S_DI
	process (SYSCLK)
	begin
		if SYSCLK'event and SYSCLK='1' then
			if RESET='1' then
				s_di_cnt <= x"A1";
			elsif S_GO='1' then
				S_DI <= s_di_cnt;
				s_di_cnt <= s_di_cnt + 1;
			else
				s_di_cnt <= s_di_cnt;
			end if;
		else
			s_di_cnt <= s_di_cnt;
		end if;
	end process;

	process
		variable ln         : line;
	begin
		-- Reset
		wait for WT_IDLE+WT_DELAY;
		RESET <= '0';

		-- Reset
		wait for WT_IDLE;
		S_SADD <= x"00000000";
		S_EADD <= x"00000013";		-- Dump 0x00-0x13
		GO <= '1';
		wait for CLK_CYCLE;
		GO <= '0';

		-- End
		write(ln, string'("--- End of Test-Bench. ---"));
		writeline(output, ln);
		wait;
	end process;

end behavior;
